// PIC16F1827 Configuration Bit Settings

// 'C' source line config statements

#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

void tmrcnt();

bit secflg;//１秒経過お知らせ用
unsigned char seccnt;//１秒カウンタ用

//割り込み処理
void interrupt intr(void) {
    TMR2IF = 0;//Timer2割り込みフラグをクリア
    tmrcnt();//１秒カウント処理へ
}

//１秒カウント処理
void tmrcnt(void){
    seccnt--;//割り込みが掛かる度に１つ減らす
    if (seccnt == 0){//０だったら１秒経過したことになる
        seccnt = 50;//8000000/4/16/250/10 = 50
        secflg = 1;//１秒経過したよ
    }
}
void DispNum(unsigned char dig, unsigned char num);

unsigned char num[] = {//Data of Digits 0-9 and .
        0b11000000,
        0b11111001,
        0b10100100,
        0b10110000,
        0b10011001,
        0b10010010,
        0b10000011,
        0b11111000,
        0b10000000,
        0b10011000,
        0b01111111
};

unsigned char dig[] = {
        0b00000001,
        0b00000010,
        0b00000100
};
//メイン処理
main(){

    OSCCON = 0b01110010;//内部クロック８MHz
    ANSELA = 0b00000000;//全てデジタルI/O
    ANSELB = 0b00000000;//全てデジタルI/O
    TRISA  = 0b00000000;//全て出力(RA5は入力のみ)
    TRISB  = 0b00000000;//全て出力

    INTCONbits.GIE = 1;//Global Interrupt Enable bit
    INTCONbits.PEIE = 1;//Peripheral Interrupt Enable bit
    PIE1bits.TMR2IE = 1;//TMR2 to PR2 match interrupt
    T2CONbits.T2OUTPS = 0b1001;//Timer2 Postscaler Select bits (1:10)
    T2CONbits.TMR2ON = 0b1;//Timer2 On bit
    T2CONbits.T2CKPS = 0b10;//Timer2-type Clock Prescale Select bits (16)
    TMR2 = 0x00;//Timer2 Module register
    PR2 = 250;//Timer2 Period register

    //１秒毎にLEDを点滅
    seccnt = 50;//8000000/4/16/250/10 = 50
                //１秒間に8000000クロック発生する(8MHz)設定にしてある
                //４つ数えるごとに１つプリスケーラに送られる
                //プリスケーラ(TxCKPS)は16回数えるとタイマー２に１つ送る
                //タイマー２は250(PR2)と値を比較して等しいとポストスケーラに１つ送る
                //ポストスケーラー(TxOUTPS)は10回数えると割り込みを発生させる
                //以上を100回行うと8000000数えたことになる
    secflg = 0;//１秒経過すると１

    unsigned int dispno = 180;

    while(1){
        if(secflg){//もし１秒経過してたら
            secflg = 0;//次の１秒までは０のまま
            dispno--;
            if(dispno == 0xFFFF) dispno = 180;
        }
        DispNum(0, (dispno%60)%10);//sec 1
        DispNum(1, (dispno%60)/10);//sec 10
        DispNum(2, dispno/60);//min
    }
}

void DispNum(unsigned char d, unsigned char n){
    LATA = dig[d];//DIG1-3
    LATB = num[n];//Digit
}